#include "ubhello.h"

void yon_load_proceed(){
        yon_config_load_config(YON_CONFIG_LOCAL,config_get_command("system"),NULL);
        yon_ubl_status_box_render(yon_char_get_localised_from_lib(LOCAL_LOAD_SUCCESS_LABEL),BACKGROUND_IMAGE_SUCCESS_TYPE);
}